@@ -15,12 +15,6 @@ class TestPluginCanHandleUrlSVTPlay(PluginCanHandleUrl):
         'https://www.svtplay.se/video/27794015/100-vaginor',
         'https://www.svtplay.se/video/28065172/motet/motet-sasong-1-det-skamtar-du-inte-om',
         'https://www.svtplay.se/dokument-inifran-att-radda-ett-barn',
-        'https://www.oppetarkiv.se/video/1399273/varfor-ar-det-sa-ont-om-q-avsnitt-1-av-5',
-        'https://www.oppetarkiv.se/video/2781201/karlekens-mirakel',
-        'https://www.oppetarkiv.se/video/10354325/studio-pop',
-        'https://www.oppetarkiv.se/video/5792180/studio-pop-david-bowie',
-        'https://www.oppetarkiv.se/video/2923832/hipp-hipp-sasong-2-avsnitt-3-av-7',
-        'https://www.oppetarkiv.se/video/3945501/cccp-hockey-cccp-hockey',
     ]
 
     should_not_match = [
