@@ -328,6 +328,37 @@ class TestTwitchHLSStream(TestMixinStreamHLS, unittest.TestCase):
             call("Waiting for pre-roll ads to finish, be patient")
         ])
 
+    @patch("streamlink.plugins.twitch.log")
+    def test_hls_low_latency_has_prefetch_disable_ads_no_preroll_with_prefetch_ads(self, mock_log):
+        # segments 3-6 are ads
+        ads = TagDateRangeAd(start=DATETIME_BASE + timedelta(seconds=3), duration=4)
+        thread, segments = self.subject([
+            # regular stream data with prefetch segments
+            Playlist(0, [Segment(0), Segment(1), SegmentPrefetch(2), SegmentPrefetch(3)]),
+            # three prefetch segments, one regular (2) and two ads (3 and 4)
+            Playlist(1, [Segment(1), SegmentPrefetch(2), ads, SegmentPrefetch(3), SegmentPrefetch(4)]),
+            # all prefetch segments are gone once regular prefetch segments have shifted
+            Playlist(2, [Segment(2), ads, Segment(3), Segment(4), Segment(5)]),
+            # still no prefetch segments while ads are playing
+            Playlist(3, [ads, Segment(3), Segment(4), Segment(5), Segment(6)]),
+            # new prefetch segments on the first regular segment occurrence
+            Playlist(4, [ads, Segment(4), Segment(5), Segment(6), Segment(7), SegmentPrefetch(8), SegmentPrefetch(9)]),
+            Playlist(5, [ads, Segment(5), Segment(6), Segment(7), Segment(8), SegmentPrefetch(9), SegmentPrefetch(10)]),
+            Playlist(6, [ads, Segment(6), Segment(7), Segment(8), Segment(9), SegmentPrefetch(10), SegmentPrefetch(11)]),
+            Playlist(7, [Segment(7), Segment(8), Segment(9), Segment(10), SegmentPrefetch(11), SegmentPrefetch(12)], end=True),
+        ], disable_ads=True, low_latency=True)
+
+        self.await_write(11)
+        content = self.await_read(read_all=True)
+        self.assertEqual(
+            content,
+            self.content(segments, cond=lambda s: 2 <= s.num <= 3 or 7 <= s.num)
+        )
+        self.assertEqual(mock_log.info.mock_calls, [
+            call("Will skip ad segments"),
+            call("Low latency streaming (HLS live edge: 2)"),
+        ])
+
     @patch("streamlink.plugins.twitch.log")
     def test_hls_low_latency_no_prefetch_disable_ads_has_preroll(self, mock_log):
         daterange = TagDateRangeAd(duration=4)
